use std::path::Path;

use calibre_db::calibre::{Calibre, RecentBooksError};
use snafu::{ResultExt, Snafu};

use crate::data::book::Book;

#[derive(Debug, Snafu)]
pub enum RecentBooksError {
    #[snafu(display("Failed to fetch recent books."))]
    RecentBooks { source: RecentBooksError },
}

pub async fn recent(calibre: &Calibre, library_path: &Path) -> Result<Vec<Book>, RecentBooksError> {
    let recent_books = calibre.recent_books(25).context(RecentBooksSnafu)?;
    let recent_books = recent_books
        .iter()
        .filter_map(|x| Book::full_book(x, calibre, library_path))
        .collect::<Vec<Book>>();

    Ok(recent_books)
}
