//! HTTP route definitions and router configuration.

use std::sync::Arc;

use axum::{response::Redirect, routing::get};
use tower_http::trace::TraceLayer;
use utoipa_axum::{router::OpenApiRouter, routes};

use crate::{
    api::{html, opds, static_files},
    app_state::AppState,
};

/// Create the main API router with all endpoints and middleware.
pub fn router(state: AppState) -> OpenApiRouter {
    let store = Arc::new(state);

    let opds_routes = OpenApiRouter::new()
        .routes(routes!(opds::books::handler))
        .routes(routes!(opds::recent::handler))
        .routes(routes!(opds::series::handler))
        .routes(routes!(opds::series::single))
        .routes(routes!(opds::authors::handler))
        .routes(routes!(opds::authors::single))
        .routes(routes!(opds::search::handler))
        .routes(routes!(opds::search::info))
        .layer(TraceLayer::new_for_http())
        .with_state(store.clone());

    let html_routes = OpenApiRouter::new()
        .route("/", get(|| async { Redirect::permanent("/recent") }))
        .routes(routes!(html::recent::handler))
        .routes(routes!(html::books::handler_init))
        .routes(routes!(html::books::handler))
        .routes(routes!(html::books::download))
        .routes(routes!(html::series::handler_init))
        .routes(routes!(html::series::handler))
        .routes(routes!(html::series::single))
        .routes(routes!(html::authors::handler_init))
        .routes(routes!(html::authors::handler))
        .routes(routes!(html::authors::single))
        .routes(routes!(html::cover::thumbnail))
        .routes(routes!(html::cover::full))
        .routes(routes!(html::search::handler))
        .routes(routes!(html::archive::handler))
        .routes(routes!(static_files::handler))
        .layer(TraceLayer::new_for_http())
        .with_state(store.clone());

    OpenApiRouter::new()
        .merge(html_routes)
        .nest("/opds", opds_routes)
}
