use std::path::Path;

use calibre_db::{calibre::Calibre, data::error::DataStoreError};

use snafu::{ResultExt, Snafu};

use crate::data::book::Book;

#[derive(Debug, Snafu)]
pub enum SearchQueryError {
    #[snafu(display("Failed to search for books."))]
    Db { source: DataStoreError },
}

pub async fn query(
    query: &str,
    calibre: &Calibre,
    library_path: &Path,
) -> Result<Vec<Book>, SearchQueryError> {
    let books = calibre
        .search(query)
        .context(DbSnafu)?
        .iter()
        .filter_map(|book| Book::full_book(book, calibre, library_path))
        .collect();

    Ok(books)
}
