use std::path::Path;

use calibre_db::{
    calibre::Calibre,
    data::{error::DataStoreError, series::Series},
};
use snafu::{ResultExt, Snafu};

use crate::data::book::Book;

#[derive(Debug, Snafu)]
pub enum SingleSeriesError {
    #[snafu(display("Failed to fetch series data."))]
    SeriesData { source: DataStoreError },
    #[snafu(display("Failed to fetch books in series."))]
    BookData { source: DataStoreError },
}

pub async fn single(
    id: u64,
    calibre: &Calibre,
    library_path: &Path,
) -> Result<(Series, Vec<Book>), SingleSeriesError> {
    let series = calibre.scalar_series(id).context(SeriesDataSnafu)?;
    let books = calibre.series_books(id).context(BookDataSnafu)?;
    let books = books
        .iter()
        .filter_map(|x| Book::full_book(x, calibre, library_path))
        .collect::<Vec<Book>>();

    Ok((series, books))
}
