use clap::Parser;
use little_hesinde::{
    RunError,
    cli::Cli,
    config::{Config, LoadError},
};
use snafu::{ResultExt, Snafu};
use tracing_subscriber::{layer::SubscriberExt, util::SubscriberInitExt};

/// Top-level application errors.
#[derive(Debug, Snafu)]
pub enum Error {
    #[snafu(display("Failed to load config."))]
    Config { source: LoadError },
    #[snafu(display("Failed to run little-hesinde."))]
    Run { source: RunError },
}

#[tokio::main]
#[snafu::report]
async fn main() -> Result<(), Error> {
    tracing_subscriber::registry()
        .with(
            tracing_subscriber::EnvFilter::try_from_default_env().unwrap_or_else(|_| {
                format!(
                    "{}=debug,tower_http=debug,axum::rejection=trace",
                    env!("CARGO_CRATE_NAME")
                )
                .into()
            }),
        )
        .with(tracing_subscriber::fmt::layer())
        .init();

    let args = Cli::parse();
    let config = Config::load(&args).context(ConfigSnafu)?;

    little_hesinde::run(config).await.context(RunSnafu)
}
