//! Error handling for OPDS data.

use std::{io, string::FromUtf8Error};

use quick_xml::SeError;
use snafu::Snafu;

/// Errors happening during handling OPDS data.
#[derive(Debug, Snafu)]
pub enum AsXmlError {
    #[snafu(display("Failed to serialize object."), visibility(pub))]
    ToString { source: SeError },
    #[snafu(display("Failed to write xml event."), visibility(pub))]
    WriteXmlEvent { source: io::Error },
    #[snafu(display("Failed to read xml event."), visibility(pub))]
    ReadXmlEvent { source: quick_xml::Error },
    #[snafu(display("Failed to read bytes as utf8 string."), visibility(pub))]
    BytesToUtf8 { source: FromUtf8Error },
}
